#!/bin/bash

# Performed by Denis Fateyev <denis@fateyev.com> for 'openssh-jail' package
# Based on `create_chroot_env` script of Oliver Meyer <o.meyer@projektfarm.de>

PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
APPS="/bin/bash /bin/cat /bin/cp /bin/egrep /bin/grep /bin/hostname /bin/ls /bin/mkdir /bin/mv /bin/ping /bin/pwd /bin/rm /bin/rmdir /bin/sh /bin/vi /usr/bin/clear /usr/bin/dircolors /usr/bin/id /usr/bin/ssh /usr/bin/sftp /usr/bin/vim /usr/openssh-jail/libexec/sftp-server"

echo " --------------------------------------------------"
echo " This script is a part of 'openssh-jail' package."
echo " It will create a jail with basic system environment."
echo -e " --------------------------------------------------\n"

# retrieve parameters
USERID="$1"
if [ -z "$USERID" ] ; then
    echo " Error: username is not defined!"
    echo -e " Usage: $0 <username> [jail-dir]\n"
    echo -e " Default jail directory: /home/<username>\n"
else
    id $USERID > /dev/null 2>&1
    if [ $? = 0 ] ; then
	echo -e " Creating the jail, please wait a minute..\n"
	
	# system directories
	JAILDIR="$2"
	if [ -z "$JAILDIR" ] ; then
	    JAILDIR=/home/$USERID
	fi   

        # we don't check if directory exists or it's permissions
        # so take care!
        cd $JAILDIR
	
	mkdir home
	ln -s / home/$USERID && chown $USERID:$USERID home/$USERID
	
	ARCH=`uname -m`
	mkdir -p bin lib usr/bin dev etc
	if [ "$ARCH" == 'x86_64' ] ; then
	    mkdir -p lib64
	fi
	
	[ ! -e dev/null ] && mknod -m 0666 dev/null c 1 3
	[ ! -e dev/zero ] && mknod -m 0666 dev/zero c 1 5
	[ ! -e dev/tty ] && mknod -m 0666 dev/tty c 5 0
	[ ! -e dev/urandom ] && mknod -m 0644 dev/urandom c 1 9
    
	# program executables
	for prog in $APPS; do

	    mkdir -p ./`dirname $prog` > /dev/null 2>&1
	    cp $prog ./$prog

	    # related libraries
	    ldd $prog > /dev/nullq
	    
	    if [ "$?" = 0 ] ; then
		LIBS=`ldd $prog | awk '{ print $3 }'`

		for l in $LIBS; do
		    mkdir -p ./`dirname $l` > /dev/null 2>&1
		    cp $l ./$l > /dev/null 2>&1
		done
	    fi
	    
	done
	
	# shared libraries
	cp /lib/libnss_compat.* /lib/libnss_files.* /lib/libnss_dns.* lib/
	cp /lib/libcom_err.* lib/
	cp /lib/libnsl.* lib/
	cp /lib/libcap.* lib/
	cp /lib/ld-linux.* lib/
	if [ "$ARCH" == 'x86_64' ] ; then
	    cp /lib64/libnss_compat.* /lib64/libnss_files.* /lib64/libnss_dns.* lib64/
	    cp /lib64/libcom_err.* lib64/
	    cp /lib64/libnsl.* lib64/
	    cp /lib64/libcap.* lib64/
	    cp /lib64/ld-linux-x86-64.* lib64/
	fi
	
	# miscellaneous
	cp -R /etc/pam.d/ etc/
	cp -R /etc/profile.d/ etc/
	cp -R /lib/security/ lib/
	if [ "$ARCH" == 'x86_64' ] ; then
	    cp -R /lib64/security/ lib64/
	fi
	cp -R /etc/security/ etc/
	cp /etc/bashrc /etc/inputrc /etc/profile etc/
	
	mkdir -p usr/share/terminfo/l usr/share/terminfo/p usr/share/terminfo/s usr/share/terminfo/v usr/share/terminfo/x
	cp /usr/share/terminfo/l/linux* usr/share/terminfo/l/
	cp /usr/share/terminfo/p/putty* usr/share/terminfo/p/
	cp /usr/share/terminfo/s/screen* usr/share/terminfo/s/
	cp /usr/share/terminfo/v/vt* usr/share/terminfo/v/
	cp /usr/share/terminfo/x/xterm* usr/share/terminfo/x/
	
	cp /etc/login.defs /etc/hosts /etc/resolv.conf etc/
	
	echo '#!/bin/bash' > usr/bin/groups
	echo "id -Gn" >> usr/bin/groups
	chmod 0755 usr/bin/groups
	
	grep /etc/passwd -e "^root" > etc/passwd
	grep /etc/passwd -e "^$USERID" >> etc/passwd
	grep /etc/group -e "^root" > etc/group
	grep /etc/group -e "^$USERID" >> etc/group
	
	# done
	echo " Username: '$USERID'; jail directory: '$JAILDIR'"
	echo -e " Complete!\n"
	
    else
	echo -e " Error: user '$USERID' is not found!\n"
    fi
fi 
